/*
    Reverse Field Dominance Filter for VirtualDub.
    Copyright (C) 2003 Donald A. Graft

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Donald Graft
	neuron2@attbi.com.
*/

#include "filter.h"

int RunProc(const FilterActivation *fa, const FilterFunctions *ff);

struct FilterDefinition filterDef =
{

    NULL, NULL, NULL,       // next, prev, module
    "reverse field dominance (1.0)",             // name
    "Reverses the field dominance of a clip.",
                            // desc
    "Donald Graft",         // maker
    NULL,                   // private_data
    0,						// inst_data_size

    NULL,                   // initProc
    NULL,                   // deinitProc
    RunProc,				// runProc
    NULL,                   // paramProc
    NULL,                   // configProc
    NULL,                   // stringProc
    NULL,					// startProc
    NULL,				    // endProc

    NULL,                   // script_obj
    NULL,                   // fssProc

};

extern "C" int __declspec(dllexport) __cdecl
VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);

extern "C" void __declspec(dllexport) __cdecl
VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd;

int __declspec(dllexport) __cdecl
VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat)
{
    if (!(fd = ff->addFilter(fm, &filterDef, sizeof(FilterDefinition))))
        return 1;
    vdfd_ver    = VIRTUALDUB_FILTERDEF_VERSION;
    vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;
    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff)
{
    ff->removeFilter(fd);
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff)
{
    PixDim w, h;
    unsigned char *src, *dst;

    src = (unsigned char *)fa->src.data + (fa->src.h - 2) * fa->src.pitch;
    dst = (unsigned char *)fa->dst.data + (fa->dst.h - 1) * fa->dst.pitch;

    for (h = 0; h < fa->src.h - 1; h++)
	{
		memcpy(dst, src, fa->src.w*4);
        src -= fa->src.pitch;
        dst -= fa->dst.pitch;
    }
	memcpy(dst, src + fa->src.pitch, fa->src.w*4);

    return 0;
}
